"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _graphqlRelay = require("graphql-relay");

Object.defineProperty(exports, "connectionFromArray", {
  enumerable: true,
  get: function get() {
    return _graphqlRelay.connectionFromArray;
  }
});
Object.defineProperty(exports, "connectionFromArraySlice", {
  enumerable: true,
  get: function get() {
    return _graphqlRelay.connectionFromArraySlice;
  }
});
Object.defineProperty(exports, "connectionFromPromisedArray", {
  enumerable: true,
  get: function get() {
    return _graphqlRelay.connectionFromPromisedArray;
  }
});
Object.defineProperty(exports, "connectionFromPromisedArraySlice", {
  enumerable: true,
  get: function get() {
    return _graphqlRelay.connectionFromPromisedArraySlice;
  }
});
Object.defineProperty(exports, "cursorForObjectInConnection", {
  enumerable: true,
  get: function get() {
    return _graphqlRelay.cursorForObjectInConnection;
  }
});
Object.defineProperty(exports, "cursorToOffset", {
  enumerable: true,
  get: function get() {
    return _graphqlRelay.cursorToOffset;
  }
});
Object.defineProperty(exports, "getOffsetWithDefault", {
  enumerable: true,
  get: function get() {
    return _graphqlRelay.getOffsetWithDefault;
  }
});
Object.defineProperty(exports, "offsetToCursor", {
  enumerable: true,
  get: function get() {
    return _graphqlRelay.offsetToCursor;
  }
});


/**
 * Copyright (c) 2017, Dirk-Jan Rutten
 * All rights reserved.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 */

/**
 * The common page info type used by all connections.
 */
var pageInfoType = "\n  # Information about pagination in a connection.'\n  type PageInfo {\n    # When paginating forwards, the cursor to continue.\n    endCursor: String\n    # When paginating forwards, are there more items?\n    hasNextPage: Boolean!\n    # When paginating backwards, are there more items?\n    hasPreviousPage: Boolean!\n    # When paginating backwards, the cursor to continue.\n    startCursor: String\n  }\n";

/**
 * Returns the type for a connection with the given name,
 * and whose nodes are of the specified type.
 */
var connectionDefinitions = function connectionDefinitions(config) {
  var name = config.name,
      edgeFields = config.edgeFields,
      connectionFields = config.connectionFields,
      nodeType = config.nodeType;


  var connectionNode = nodeType ? nodeType : name;

  var connectionType = "\n    # An edge in a connection.\n    type " + name + "Edge {\n      # A cursor for use in pagination.\n      cursor: String!\n      # The item at the end of the edge\n      node: " + connectionNode + "\n      " + (edgeFields ? edgeFields : "") + "\n    }\n  \n    # A connection to a list of items.\n    type " + name + "Connection {\n      # A list of edges.\n      edges: [" + name + "Edge]\n      # Information to aid in pagination.\n      pageInfo: PageInfo!\n      " + (connectionFields ? connectionFields : "") + "\n    }\n  ";

  return { connectionType: connectionType };
};

var flattenArgs = function flattenArgs(additionalArgs) {
  var args = "";
  if (additionalArgs) {
    args = additionalArgs.reduce(function (acc, cur) {
      return acc + " " + cur + ",";
    }, ",");
    args = args.slice(0, -1);
  }
  return args;
};

/*
 * Returns the arguments appropriate to include on a field
 * whose return type is a connection type with forward pagination.
 *
 * Additional fields may be passed as an argument. These 
 * will be added to the connection argument.
 */
var connectionArgs = function connectionArgs(additionalArgs) {
  var args = "";
  if (additionalArgs) {
    args = flattenArgs(additionalArgs);
  }

  return "(\n    # Returns the first n elements from the list.\n    first: Int, \n    # Returns the elements in the list that come after the specified cursor.\n    after: String, \n    # Returns the last n elements from the list.\n    last: Int, \n    # Returns the elements in the list that come before the specified cursor.\n    before: String" + args + ")";
};

/*
 * Returns the arguments appropriate to include on a field
 * whose return type is a connection type with forward pagination.
 * 
 * Additional fields may be passed as an argument. These 
 * will be added to the connection argument.
 */
var forwardConnectionArgs = function forwardConnectionArgs(additionalArgs) {
  var args = "";
  if (additionalArgs) {
    args = flattenArgs(additionalArgs);
  }
  return "(first: Int, after: String" + args + ")";
};

/*
 * Returns the arguments appropriate to include on a field
 * whose return type is a connection type with backwards pagination.
 * 
 * Additional fields may be passed as an argument. These 
 * will be added to the connection argument.
 */
var backwardConnectionArgs = function backwardConnectionArgs(additionalArgs) {
  var args = "";
  if (additionalArgs) {
    args = flattenArgs(additionalArgs);
  }
  return "(last: Int, before: String" + args + ")";
};

exports.pageInfoType = pageInfoType;
exports.connectionDefinitions = connectionDefinitions;
exports.connectionArgs = connectionArgs;
exports.forwardConnectionArgs = forwardConnectionArgs;
exports.backwardConnectionArgs = backwardConnectionArgs;

// Expose some helpful utilities directly from graphql-relay.