"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.toGlobalId = exports.fromGlobalId = exports.nodeDefinitions = exports.globalIdResolver = exports.nodesField = exports.nodeField = exports.nodeInterface = undefined;

var _graphqlRelay = require("graphql-relay");

Object.defineProperty(exports, "fromGlobalId", {
  enumerable: true,
  get: function get() {
    return _graphqlRelay.fromGlobalId;
  }
});
Object.defineProperty(exports, "toGlobalId", {
  enumerable: true,
  get: function get() {
    return _graphqlRelay.toGlobalId;
  }
});


/**
 * This is the Node interface definition.
 */


/**
 * Copyright (c) 2017, Dirk-Jan Rutten
 * All rights reserved.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 */

var nodeInterface = "\n  # An object with an ID\n  interface Node {\n    # The id of the object.\n    id: ID!\n  }\n";

/**
 * This is the node field that should be added
 * to the root Query type.
 */
var nodeField = "\n  # Fetches an object given its ID\n  node(\n    # The ID of an object\n    id: ID!\n  ): Node\n";

/**
 * This is the nodes field that should be added
 * to the root Query type.
 */
var nodesField = "\n  # Fetches objects given their IDs\n  nodes(\n    # The IDs of objects\n    ids: [ID!]!\n  ): [Node]! \n";

/**
 * Creates the resolver for an id field on a node, using `toGlobalId` to
 * construct the ID from the provided typename. The type-specific ID is fetched
 * by calling idFetcher on the object, or if not provided, by accessing the `id`
 * property on the object.
 */
function globalIdResolver(typeName, idFetcher) {
  return function (obj, args, context, info) {
    return (0, _graphqlRelay.toGlobalId)(typeName || info.parentType.name, idFetcher ? idFetcher(obj, context, info) : obj.id);
  };
}

/**
 * Given a function to map from an ID to an underlying object it creates
 * the resolvers for the `node` and `nodes` root field.
 */
function nodeDefinitions(idFetcher) {
  return {
    nodeResolver: function nodeResolver(obj, _ref, context, info) {
      var id = _ref.id;
      return idFetcher(id, context, info);
    },
    nodesResolver: function nodesResolver(obj, _ref2, context, info) {
      var ids = _ref2.ids;
      return Promise.all(ids.map(function (id) {
        return Promise.resolve(idFetcher(id, context, info));
      }));
    }
  };
}

exports.nodeInterface = nodeInterface;
exports.nodeField = nodeField;
exports.nodesField = nodesField;
exports.globalIdResolver = globalIdResolver;
exports.nodeDefinitions = nodeDefinitions;